@@ -1,22 +1,29 @@
-# coding: utf-8
-import os
+import glob
 import importlib
+import os
 from contextlib import contextmanager
-
-from six import string_types
+from contextlib import suppress
 
 from dynaconf import default_settings
-from dynaconf.loaders import (
-    default_loader,
-    settings_loader,
-    yaml_loader,
-    enable_external_loaders
-)
-from dynaconf.utils.functional import LazyObject, empty
-from dynaconf.utils.parse_conf import converters, parse_conf_data, true_values
-from dynaconf.utils import BANNER, compat_kwargs, raw_logger
-from dynaconf.validator import ValidatorList
+from dynaconf.loaders import default_loader
+from dynaconf.loaders import enable_external_loaders
+from dynaconf.loaders import settings_loader
+from dynaconf.loaders import yaml_loader
+from dynaconf.utils import BANNER
+from dynaconf.utils import compat_kwargs
+from dynaconf.utils import deduplicate
+from dynaconf.utils import ensure_a_list
+from dynaconf.utils import missing
+from dynaconf.utils import object_merge
+from dynaconf.utils import raw_logger
 from dynaconf.utils.boxing import DynaBox
+from dynaconf.utils.files import find_file
+from dynaconf.utils.functional import empty
+from dynaconf.utils.functional import LazyObject
+from dynaconf.utils.parse_conf import converters
+from dynaconf.utils.parse_conf import parse_conf_data
+from dynaconf.utils.parse_conf import true_values
+from dynaconf.validator import ValidatorList
 
 
 class LazySettings(LazyObject):
@@ -28,9 +35,9 @@ class LazySettings(LazyObject):
 
     Then when you first access a value, this will be set up and loaders will
     be executes looking for default config files or the file defined in
-    SETTINGS_MODULE_FOR_DYNACONF variable::
+    SETTINGS_FILE_FOR_DYNACONF variable::
 
-        >>> settings.SETTINGS_MODULE
+        >>> settings.SETTINGS_FILE_FOR_DYNACONF
 
     Or when you call::
 
@@ -38,7 +45,7 @@ class LazySettings(LazyObject):
 
     You can define in your settings module a list of loaders to get values
     from different stores. By default it will try environment variables
-    starting with GLOBAL_ENV_FOR_DYNACONF (by defaulf `DYNACONF_`)
+    starting with ENVVAR_PREFIX_FOR_DYNACONF (by defaulf `DYNACONF_`)
 
     You can also import this directly and customize it.
     in a file `proj/conf.py`::
@@ -77,8 +84,6 @@ class LazySettings(LazyObject):
         """
         compat_kwargs(kwargs)
         self._kwargs = kwargs
-        for k, v in kwargs.items():
-            setattr(default_settings, k.upper(), v)
         super(LazySettings, self).__init__()
 
     def __getattr__(self, name):
@@ -91,10 +96,12 @@ class LazySettings(LazyObject):
                 "or belongs to different env" % name
             )
         if (
-            name.isupper() and
-            (self._wrapped._fresh or
-             name in self._wrapped.FRESH_VARS_FOR_DYNACONF) and
-            name not in dir(default_settings)
+            name.isupper()
+            and (
+                self._wrapped._fresh
+                or name in self._wrapped.FRESH_VARS_FOR_DYNACONF
+            )
+            and name not in dir(default_settings)
         ):
             return self._wrapped.get_fresh(name)
         return getattr(self._wrapped, name)
@@ -107,11 +114,15 @@ class LazySettings(LazyObject):
 
     def _setup(self):
         """Initial setup, run once."""
-        environment_variable = default_settings.ENVVAR_FOR_DYNACONF
+        default_settings.reload()
+        environment_variable = self._kwargs.get(
+            "ENVVAR_FOR_DYNACONF", default_settings.ENVVAR_FOR_DYNACONF
+        )
         settings_module = os.environ.get(environment_variable)
         self._wrapped = Settings(
             settings_module=settings_module, **self._kwargs
         )
+        self.logger.debug("Lazy Settings _setup ...")
 
     def configure(self, settings_module=None, **kwargs):
         """
@@ -121,9 +132,15 @@ class LazySettings(LazyObject):
         :param settings_module: defines the setttings file
         :param kwargs:  override default settings
         """
+        default_settings.reload()
+        environment_var = self._kwargs.get(
+            "ENVVAR_FOR_DYNACONF", default_settings.ENVVAR_FOR_DYNACONF
+        )
+        settings_module = settings_module or os.environ.get(environment_var)
         compat_kwargs(kwargs)
         kwargs.update(self._kwargs)
         self._wrapped = Settings(settings_module=settings_module, **kwargs)
+        self.logger.debug("Lazy Settings configured ...")
 
     @property
     def configured(self):
@@ -147,6 +164,7 @@ class Settings(object):
         self._logger = None
         self._fresh = False
         self._loaded_envs = []
+        self._loaded_files = []
         self._deleted = set()
         self._store = {}
         self._loaded_by_loaders = {}
@@ -155,14 +173,16 @@ class Settings(object):
         self.environ = os.environ
         self.SETTINGS_MODULE = None
         self._not_installed_warnings = []
+        self._memoized = None
 
         compat_kwargs(kwargs)
         if settings_module:
-            self.set('SETTINGS_MODULE', settings_module)
+            self.set("SETTINGS_FILE_FOR_DYNACONF", settings_module)
         for key, value in kwargs.items():
             self.set(key, value)
         # execute loaders only after setting defaults got from kwargs
         self._defaults = kwargs
+        self.logger.debug("Initializing Dynaconf (%s)", self._store)
         self.execute_loaders()
 
     def __call__(self, *args, **kwargs):
@@ -186,11 +206,15 @@ class Settings(object):
         if hasattr(self, name):
             super(Settings, self).__delattr__(name)
 
+    def __contains__(self, item):
+        "Respond to `item in settings`"
+        return item in self.store
+
     def __getitem__(self, item):
         """Allow getting variables as dict keys `settings['KEY']`"""
         value = self.get(item)
         if value is None:
-            raise KeyError('{0} does not exists'.format(item))
+            raise KeyError("{0} does not exists".format(item))
         return value
 
     def __setitem__(self, key, value):
@@ -210,7 +234,42 @@ class Settings(object):
         """Redirects to store object"""
         return self.store.values()
 
-    def get(self, key, default=None, cast=None, fresh=False):
+    def as_dict(self, env=None, internal=False):
+        """Returns a dictionary with set key and values.
+
+        :param env: Str env name, default self.current_env `DEVELOPMENT`
+        :param internal: bool - should include dynaconf internal vars?
+        """
+        ctx_mgr = suppress() if env is None else self.using_env(env)
+        with ctx_mgr:
+            data = self.store.copy()
+            # if not internal remove internal settings
+            if not internal:
+                for name in dir(default_settings):
+                    data.pop(name, None)
+            return data
+
+    def _dotted_get(self, dotted_key, default=None, **kwargs):
+        """
+        Perform dotted key lookups and keep track of where we are.
+        """
+        split_key = dotted_key.split(".")
+        name, keys = split_key[0], split_key[1:]
+        result = self.get(name, default=default, **kwargs)
+        self._memoized = result
+
+        # If we've reached the end, then return result and clear the
+        # memoized data.
+        if not keys or result is default:
+            self._memoized = None
+            return result
+
+        # If we've still got key elements to traverse, let's do that.
+        return self._dotted_get(".".join(keys), default=default, **kwargs)
+
+    def get(
+        self, key, default=None, cast=None, fresh=False, dotted_lookup=True
+    ):
         """
         Get a value from settings store, this is the prefered way to access::
 
@@ -221,19 +280,29 @@ class Settings(object):
         :param default: In case of not found it will be returned
         :param cast: Should cast in to @int, @float, @bool or @json ?
         :param fresh: Should reload from loaders store before access?
+        :param dotted_lookup: Should perform dotted-path lookup?
         :return: The value if found, default or None
         """
         key = key.upper()
+
+        if "." in key and dotted_lookup:
+            return self._dotted_get(
+                dotted_key=key, default=default, cast=cast, fresh=fresh
+            )
+
         if key in self._deleted:
             return default
 
         if (
-            (fresh or self._fresh or key in self.FRESH_VARS_FOR_DYNACONF) and
-            key not in dir(default_settings)
-        ):
+            fresh
+            or self._fresh
+            or key in getattr(self, "FRESH_VARS_FOR_DYNACONF", ())
+        ) and key not in dir(default_settings):
             self.unset(key)
             self.execute_loaders(key=key)
-        data = self.store.get(key, default)
+
+        store = self._memoized or self.store
+        data = store.get(key, default)
         if cast:
             data = converters.get(cast)(data)
         return data
@@ -248,12 +317,7 @@ class Settings(object):
         key = key.upper()
         if key in self._deleted:
             return False
-        if (
-            (fresh or self._fresh or key in self.FRESH_VARS_FOR_DYNACONF) and
-            key not in dir(default_settings)
-        ):
-            self.execute_loaders(key=key)
-        return key in self.store
+        return self.get(key, fresh=fresh, default=missing) is not missing
 
     def get_fresh(self, key, default=None, cast=None):
         """This is a shortcut to `get(key, fresh=True)`. always reload from
@@ -290,19 +354,19 @@ class Settings(object):
 
     def as_bool(self, key):
         """Partial method for get with bool cast"""
-        return self.get(key, cast='@bool')
+        return self.get(key, cast="@bool")
 
     def as_int(self, key):
         """Partial method for get with int cast"""
-        return self.get(key, cast='@int')
+        return self.get(key, cast="@int")
 
     def as_float(self, key):
         """Partial method for get with float cast"""
-        return self.get(key, cast='@float')
+        return self.get(key, cast="@float")
 
     def as_json(self, key):
         """Partial method for get with json cast"""
-        return self.get(key, cast='@json')
+        return self.get(key, cast="@json")
 
     @property
     def logger(self):  # pragma: no cover
@@ -400,20 +464,26 @@ class Settings(object):
             return self.loaded_envs[-1]
         except IndexError:
             return self.ENV_FOR_DYNACONF
+
     # compat
     current_namespace = current_env
 
     @property
     def settings_module(self):
         """Gets SETTINGS_MODULE variable"""
-        settings_module = os.environ.get(
-            self.ENVVAR_FOR_DYNACONF,
-            self.SETTINGS_MODULE_FOR_DYNACONF
+        settings_module = parse_conf_data(
+            os.environ.get(
+                self.ENVVAR_FOR_DYNACONF, self.SETTINGS_FILE_FOR_DYNACONF
+            ),
+            tomlfy=True,
         )
-        if settings_module != self.SETTINGS_MODULE:
-            self.set('SETTINGS_MODULE', settings_module)
+        if settings_module != getattr(self, "SETTINGS_MODULE", None):
+            self.set("SETTINGS_MODULE", settings_module)
         return self.SETTINGS_MODULE
 
+    # Backwards compatibility see #169
+    settings_file = settings_module
+
     def setenv(self, env=None, clean=True, silent=True, filename=None):
         """Used to interactively change the env
         Example of settings.toml::
@@ -440,10 +510,10 @@ class Settings(object):
         """
         env = env or self.ENV_FOR_DYNACONF
 
-        if not isinstance(env, string_types):
-            raise AttributeError('env should be a string')
+        if not isinstance(env, str):
+            raise AttributeError("env should be a string")
         if "_" in env:
-            raise AttributeError('env should not contains _')
+            raise AttributeError("env should not contains _")
 
         self.logger.debug("env switching to: %s", env)
 
@@ -473,7 +543,7 @@ class Settings(object):
         """
         key = key.strip().upper()
         if key not in dir(default_settings) and key not in self._defaults:
-            self.logger.debug('Unset %s', key)
+            self.logger.debug("Unset %s", key)
             delattr(self, key)
             self.store.pop(key, None)
 
@@ -485,19 +555,52 @@ class Settings(object):
         for key in keys:
             self.unset(key)
 
-    def set(self, key, value, loader_identifier=None, tomlfy=False):
+    def _dotted_set(self, dotted_key, value, tomlfy=False, **kwargs):
+
+        data = DynaBox(default_box=True)
+        tree = data
+        split_keys = dotted_key.split(".")
+
+        for k in split_keys[:-1]:
+            tree = tree.setdefault(k, {})
+
+        value = parse_conf_data(value, tomlfy=tomlfy)
+        tree[split_keys[-1]] = value
+
+        self.update(data=data, **kwargs)
+
+    def set(
+        self,
+        key,
+        value,
+        loader_identifier=None,
+        tomlfy=False,
+        dotted_lookup=True,
+        is_secret=False,
+    ):
         """Set a value storing references for the loader
 
         :param key: The key to store
         :param value: The value to store
         :param loader_identifier: Optional loader name e.g: toml, yaml etc.
         :param tomlfy: Bool define if value is parsed by toml (defaults False)
+        :param is_secret: Bool define if secret values is hidden on logs.
         """
+
+        if "." in key and dotted_lookup is True:
+            return self._dotted_set(
+                key, value, loader_identifier=loader_identifier, tomlfy=tomlfy
+            )
+
         value = parse_conf_data(value, tomlfy=tomlfy)
+        key = key.strip().upper()
+        existing = getattr(self, key, None)
+        if existing is not None and existing != value:
+            value = self._merge_before_set(key, existing, value, is_secret)
+
         if isinstance(value, dict):
             value = DynaBox(value, box_it_up=True)
 
-        key = key.strip().upper()
         setattr(self, key, value)
         self.store[key] = value
         self._deleted.discard(key)
@@ -512,8 +615,14 @@ class Settings(object):
             # a default value and goes away only when explicitly unset
             self._defaults[key] = value
 
-    def update(self, data=None, loader_identifier=None,
-               tomlfy=False, **kwargs):
+    def update(
+        self,
+        data=None,
+        loader_identifier=None,
+        tomlfy=False,
+        is_secret=False,
+        **kwargs
+    ):
         """
         Update values in the current settings object without saving in stores::
 
@@ -535,14 +644,79 @@ class Settings(object):
         data = data or {}
         data.update(kwargs)
         for key, value in data.items():
-            self.set(key, value, loader_identifier=loader_identifier,
-                     tomlfy=tomlfy)
+            self.set(
+                key,
+                value,
+                loader_identifier=loader_identifier,
+                tomlfy=tomlfy,
+                is_secret=is_secret,
+            )
+
+    def _merge_before_set(self, key, existing, value, is_secret):
+        """Merge the new value being set with the existing value before set"""
+
+        def _log_before_merging(_value):
+            self.logger.debug(
+                "Merging existing %s: %s with new: %s", key, existing, _value
+            )
+
+        def _log_after_merge(_value):
+            self.logger.debug("%s merged to %s", key, _value)
+
+        global_merge = getattr(self, "MERGE_ENABLED_FOR_DYNACONF", False)
+
+        if isinstance(value, dict):
+            local_merge = value.pop(
+                "dynaconf_merge", value.pop("dynaconf_merge_unique", None)
+            )
+            if global_merge or local_merge:
+                safe_value = {k: "***" for k in value} if is_secret else value
+                _log_before_merging(safe_value)
+                object_merge(existing, value)
+                safe_value = (
+                    {
+                        k: ("***" if k in safe_value else v)
+                        for k, v in value.items()
+                    }
+                    if is_secret
+                    else value
+                )
+                _log_after_merge(safe_value)
+
+        if isinstance(value, (list, tuple)):
+            local_merge = (
+                "dynaconf_merge" in value or "dynaconf_merge_unique" in value
+            )
+            if global_merge or local_merge:
+                value = list(value)
+                unique = False
+
+                if local_merge:
+                    try:
+                        value.remove("dynaconf_merge")
+                    except ValueError:  # EAFP
+                        value.remove("dynaconf_merge_unique")
+                        unique = True
+
+                original = set(value)
+                _log_before_merging(
+                    ["***" for item in value] if is_secret else value
+                )
+                object_merge(existing, value, unique=unique)
+                safe_value = (
+                    ["***" if item in original else item for item in value]
+                    if is_secret
+                    else value
+                )
+                _log_after_merge(safe_value)
+
+        return value
 
     @property
     def loaders(self):  # pragma: no cover
         """Return available loaders"""
         if self.LOADERS_FOR_DYNACONF in (None, 0, "0", "false", False):
-            self.logger.info('No loader defined')
+            self.logger.info("No loader defined")
             return []
 
         if not self._loaders:
@@ -563,49 +737,116 @@ class Settings(object):
         :param env: The environment to load
         :param silent: If loading erros is silenced
         :param key: if provided load a single key
-        :param filename: optinal custom filename to load
+        :param filename: optional custom filename to load
         """
         if key is None:
             default_loader(self, self._defaults)
+        env = (env or self.current_env).upper()
         silent = silent or self.SILENT_ERRORS_FOR_DYNACONF
-        settings_loader(self, env=env, silent=silent, key=key,
-                        filename=filename)
+        settings_loader(
+            self, env=env, silent=silent, key=key, filename=filename
+        )
         self.load_extra_yaml(env, silent, key)  # DEPRECATED
         enable_external_loaders(self)
         for loader in self.loaders:
-            self.logger.debug('Dynaconf executing: %s', loader.__name__)
+            self.logger.debug("Dynaconf executing: %s", loader.__name__)
             loader.load(self, env, silent=silent, key=key)
+        self.load_includes(env, silent=silent, key=key)
+        self.logger.debug("Loaded Files: %s", deduplicate(self._loaded_files))
+
+    def load_includes(self, env, silent, key):
+        """Do we have any nested includes we need to process?"""
+        includes = self.get("DYNACONF_INCLUDE", [])
+        includes.extend(ensure_a_list(self.get("INCLUDES_FOR_DYNACONF")))
+        if includes:
+            self.logger.debug("Processing includes %s", includes)
+            self.load_file(path=includes, env=env, silent=silent, key=key)
+
+    def load_file(self, path=None, env=None, silent=True, key=None):
+        """Programmatically load files from ``path``.
+
+        :param path: A single filename or a file list
+        :param env: Which env to load from file (default current_env)
+        :param silent: Should raise errors?
+        :param key: Load a single key?
+        """
+        env = (env or self.current_env).upper()
+        files = ensure_a_list(path)
+        if files:
+            self.logger.debug("Got %s files to process", files)
+            already_loaded = set()
+            for _filename in files:
+                self.logger.debug("Processing file %s", _filename)
+                filepath = os.path.join(self._root_path, _filename)
+                self.logger.debug("File path is %s", filepath)
+                # Handle possible *.globs sorted alphanumeric
+                for path in sorted(glob.glob(filepath)):
+                    self.logger.debug("Loading %s", path)
+                    if path in already_loaded:  # pragma: no cover
+                        self.logger.debug("Skipping %s, already loaded", path)
+                        continue
+                    settings_loader(
+                        obj=self,
+                        env=env,
+                        silent=silent,
+                        key=key,
+                        filename=path,
+                    )
+                    already_loaded.add(path)
+            if not already_loaded:
+                self.logger.warning(
+                    "Not able to locate the files %s to load", files
+                )
+
+    @property
+    def _root_path(self):
+        """ROOT_PATH_FOR_DYNACONF or the path of first loaded file or '.'"""
+        if self.ROOT_PATH_FOR_DYNACONF is not None:
+            return self.ROOT_PATH_FOR_DYNACONF
+        elif self._loaded_files:  # called once
+            root_path = os.path.dirname(self._loaded_files[0])
+            self.set("ROOT_PATH_FOR_DYNACONF", root_path)
+            return root_path
 
     def load_extra_yaml(self, env, silent, key):
         """This is deprecated, kept for compat
 
         .. deprecated:: 1.0.0
-            Use multiple settings files instead.
+            Use multiple settings or INCLUDES_FOR_DYNACONF files instead.
         """
-        if self.get('YAML') is not None:
+        if self.get("YAML") is not None:
             self.logger.warning(
                 "The use of YAML var is deprecated, please define multiple "
                 "filepaths instead: "
-                "e.g: SETTINGS_MODULE_FOR_DYNACONF = "
-                "'settings.py,settings.yaml,settings.toml'"
+                "e.g: SETTINGS_FILE_FOR_DYNACONF = "
+                "'settings.py,settings.yaml,settings.toml' or "
+                "INCLUDES_FOR_DYNACONF=['path.toml', 'folder/*']"
             )
             yaml_loader.load(
-                self, env=env,
-                filename=self.get('YAML'),
+                self,
+                env=env,
+                filename=self.find_file(self.get("YAML")),
                 silent=silent,
-                key=key
+                key=key,
             )
 
     def path_for(self, *args):
-        """Path containing PROJECT_ROOT_FOR_DYNACONF"""
-        if args and args[0].startswith('/'):
+        """Path containing _root_path"""
+        if args and args[0].startswith(os.path.sep):
             return os.path.join(*args)
-        return os.path.join(self.PROJECT_ROOT_FOR_DYNACONF, *args)
+        return os.path.join(self._root_path or os.getcwd(), *args)
+
+    def find_file(self, *args, **kwargs):
+        kwargs.setdefault("project_root", self._root_path)
+        kwargs.setdefault(
+            "skip_files", self.get("SKIP_FILES_FOR_DYNACONF", [])
+        )
+        return find_file(*args, **kwargs)
 
     @property
     def validators(self):
         """Gets or creates validator wrapper"""
-        if not hasattr(self, '_validators'):
+        if not hasattr(self, "_validators"):
             self._validators = ValidatorList(self)
         return self._validators
 
@@ -629,7 +870,16 @@ class Settings(object):
         :param key: The flag name
         :param env: The env to look for
         """
-        env = env or self.GLOBAL_ENV_FOR_DYNACONF
+        env = env or self.ENVVAR_PREFIX_FOR_DYNACONF or "DYNACONF"
         with self.using_env(env):
             value = self.get_fresh(key)
             return value is True or value in true_values
+
+    def populate_obj(self, obj, keys=None):
+        """Given the `obj` populate it using self.store items."""
+        keys = keys or self.keys()
+        for key in keys:
+            key = key.upper()
+            value = self.get(key, empty)
+            if value is not empty:
+                setattr(obj, key, value)
