@@ -1,5 +1,7 @@
-# coding: utf-8
-from dynaconf.utils.files import find_file
+import io
+import os
+
+from dynaconf.utils import ensure_a_list
 from dynaconf.utils import raw_logger
 
 
@@ -18,8 +20,9 @@ class BaseLoader(object):
     :param string_reader: {[callable]} -- [reads string return dict]
     """
 
-    def __init__(self, obj, env, identifier, extensions,
-                 file_reader, string_reader):
+    def __init__(
+        self, obj, env, identifier, extensions, file_reader, string_reader
+    ):
         """Instantiates a loader for different sources"""
         self.obj = obj
         self.env = env or obj.current_env
@@ -34,7 +37,7 @@ class BaseLoader(object):
             logger.warning(
                 "%(ident)s support is not installed in your environment. "
                 "`pip install dynaconf[%(ident)s]`",
-                {'ident': identifier}
+                {"ident": identifier},
             )
         obj._not_installed_warnings.append(identifier)
 
@@ -51,20 +54,22 @@ class BaseLoader(object):
             return
 
         if not isinstance(filename, (list, tuple)):
-            split_files = filename.split(',')
+            split_files = ensure_a_list(filename)
             if all([f.endswith(self.extensions) for f in split_files]):  # noqa
-                files = split_files  # it is a ['file.ini', ...]
-            else:  # it is a single ini string
+                files = split_files  # it is a ['file.ext', ...]
+            else:  # it is a single config as string
                 files = [filename]
         else:  # it is already a list/tuple
             files = filename
 
+        self.obj._loaded_files.extend(files)
+
         # add the [default] env
-        env_list = [self.obj.get('DEFAULT_ENV_FOR_DYNACONF')]
+        env_list = [self.obj.get("DEFAULT_ENV_FOR_DYNACONF")]
 
         # compatibility with older versions that still uses [dynaconf] as
         # [default] env
-        global_env = self.obj.get('GLOBAL_ENV_FOR_DYNACONF')
+        global_env = self.obj.get("ENVVAR_PREFIX_FOR_DYNACONF") or "DYNACONF"
         if global_env not in env_list:
             env_list.append(global_env)
 
@@ -73,24 +78,30 @@ class BaseLoader(object):
             env_list.append(self.env)
 
         # add the [global] env
-        env_list.append('GLOBAL')
+        env_list.append("GLOBAL")
 
         # load all envs
         self._read(files, env_list, silent, key)
 
     def _read(self, files, envs, silent=True, key=None):
         for source_file in files:
-            if source_file.endswith(self.extensions):  # pragma: no cover
+            if source_file.endswith(self.extensions):
                 try:
-                    source_data = self.file_reader(
-                        open(find_file(source_file))
+                    with io.open(
+                        source_file,
+                        encoding=self.obj.get(
+                            "ENCODING_FOR_DYNACONF", "utf-8"
+                        ),
+                    ) as open_file:
+                        source_data = self.file_reader(open_file)
+                    self.obj.logger.debug(
+                        "{}_loader: {}".format(self.identifier, source_file)
                     )
-                    self.obj.logger.debug('{}_loader: {}'.format(
-                        self.identifier, source_file))
                 except IOError:
                     self.obj.logger.debug(
-                        '{}_loader: {} (Ignored, file not Found)'.format(
-                            self.identifier, source_file)
+                        "{}_loader: {} (Ignored, file not Found)".format(
+                            self.identifier, source_file
+                        )
                     )
                     source_data = None
             else:
@@ -100,10 +111,9 @@ class BaseLoader(object):
             if not source_data:
                 continue
 
+            # env name is checked in lower
             source_data = {
-                k.lower(): value
-                for k, value
-                in source_data.items()
+                k.lower(): value for k, value in source_data.items()
             }
 
             for env in envs:
@@ -112,31 +122,51 @@ class BaseLoader(object):
                 try:
                     data = source_data[env.lower()]
                 except KeyError:
-                    if env not in (self.obj.get('GLOBAL_ENV_FOR_DYNACONF'),
-                                   'GLOBAL'):
-                        message = '%s_loader: %s env not defined in %s' % (
-                            self.identifier, env, source_file)
+                    if env not in (
+                        self.obj.get("ENVVAR_PREFIX_FOR_DYNACONF")
+                        or "DYNACONF",
+                        "GLOBAL",
+                    ):
+                        message = "%s_loader: %s env not defined in %s" % (
+                            self.identifier,
+                            env,
+                            source_file,
+                        )
                         if silent:
                             self.obj.logger.warning(message)
                         else:
                             raise KeyError(message)
                     continue
 
-                if env != self.obj.get('DEFAULT_ENV_FOR_DYNACONF'):
+                if env != self.obj.get("DEFAULT_ENV_FOR_DYNACONF"):
                     identifier = "{0}_{1}".format(self.identifier, env.lower())
                 else:
                     identifier = self.identifier
 
-                if not key:
-                    self.obj.update(data, loader_identifier=identifier)
-                elif key in data:
-                    self.obj.set(key, data.get(key),
-                                 loader_identifier=identifier)
+                # data 1st level keys should be transformed to upper case.
+                data = {k.upper(): v for k, v in data.items()}
+                if key:
+                    key = key.upper()
+
+                is_secret = "secret" in source_file
 
                 self.obj.logger.debug(
-                    '{}_loader: {}: {}'.format(
+                    "{}_loader: {}[{}]{}".format(
                         self.identifier,
+                        os.path.split(source_file)[-1],
                         env.lower(),
-                        list(data.keys()) if 'secret' in source_file else data
+                        list(data.keys()) if is_secret else data,
                     )
                 )
+
+                if not key:
+                    self.obj.update(
+                        data, loader_identifier=identifier, is_secret=is_secret
+                    )
+                elif key in data:
+                    self.obj.set(
+                        key,
+                        data.get(key),
+                        loader_identifier=identifier,
+                        is_secret=is_secret,
+                    )
