@@ -1,7 +1,11 @@
-# coding: utf-8
-from dynaconf.loaders.base import BaseLoader
+import io
+from pathlib import Path
+
+from dynaconf import default_settings
 from dynaconf.constants import INI_EXTENSIONS
-from dynaconf.utils import dictmerge
+from dynaconf.loaders.base import BaseLoader
+from dynaconf.utils import object_merge
+
 try:
     from configobj import ConfigObj
 except ImportError as e:  # pragma: no cover
@@ -20,16 +24,16 @@ def load(obj, env=None, silent=True, key=None, filename=None):
     :return: None
     """
     if ConfigObj is None:  # pragma: no cover
-        BaseLoader.warn_not_installed(obj, 'ini')
+        BaseLoader.warn_not_installed(obj, "ini")
         return
 
     loader = BaseLoader(
         obj=obj,
         env=env,
-        identifier='ini',
+        identifier="ini",
         extensions=INI_EXTENSIONS,
         file_reader=lambda fileobj: ConfigObj(fileobj).dict(),
-        string_reader=lambda strobj: ConfigObj(strobj.split('\n')).dict()
+        string_reader=lambda strobj: ConfigObj(strobj.split("\n")).dict(),
     )
     loader.load(filename=filename, key=key, silent=silent)
 
@@ -41,11 +45,12 @@ def write(settings_path, settings_data, merge=True):
     :param settings_data: a dictionary with data
     :param merge: boolean if existing file should be merged with new data
     """
+    settings_path = Path(settings_path)
     if settings_path.exists() and merge:  # pragma: no cover
-        settings_data = dictmerge(
-            ConfigObj(open(str(settings_path))).dict(),
-            settings_data
-        )
+        with io.open(
+            str(settings_path), encoding=default_settings.ENCODING_FOR_DYNACONF
+        ) as open_file:
+            object_merge(ConfigObj(open_file).dict(), settings_data)
     new = ConfigObj()
     new.update(settings_data)
-    new.write(open(str(settings_path), 'bw'))
+    new.write(open(str(settings_path), "bw"))
