@@ -1,5 +1,3 @@
-# coding: utf-8
-
 from box import Box
 
 
@@ -21,6 +19,12 @@ class DynaBox(Box):
             n_item = item.lower() if item.isupper() else item.upper()
             return super(DynaBox, self).__getitem__(n_item, *args, **kwargs)
 
+    def __copy__(self):
+        return self.__class__(super(Box, self).copy())
+
+    def copy(self):
+        return self.__class__(super(Box, self).copy())
+
     def get(self, item, default=None, *args, **kwargs):
         value = super(DynaBox, self).get(item, default, *args, **kwargs)
         if value is None or value == default:
